// const http = require('http')
// const https = require('https')

const client = require('prom-client')
// const config = require('config')
// const URL = require('url')
const pkg = require('../package.json')

const yves = require('yves')
// const debugMetrics = yves.debugger(`${pkg.name.replace(/-/g,':')}:metrics`)

const debugReport = yves.debugger(`${pkg.name.replace(/-/g,':')}:report`)

const sortobject = require('deep-sort-object')
const util = require('./util')


// if (config.metrics.enabled) {
//   http.request = wrapper(http)
//   https.request = wrapper(https)
// }

const metric = {
  web: {
    requests: {
      duration: new client.Summary({ name: 'node_yonderbox_graphql_web_request_duration_milliseconds', help: 'web request duration in milliseconds', labelNames: ['path', 'method', 'status'] }),
      buckets: new client.Histogram({ name: 'node_yonderbox_graphql_web_request_buckets_milliseconds', help: 'web request duration buckets in milliseconds. Bucket size set to 500 and 2000 ms to enable apdex calculations with a T of 300ms', labelNames: ['path', 'method', 'status'], buckets: [ 500, 2000 ] })
    }
  },
  api: {
    requests: {
      duration: new client.Summary({ name: 'node_yonderbox_graphql_api_request_duration_milliseconds', help: 'api request duration in milliseconds', labelNames: ['url', 'method', 'status'] }),
      buckets: new client.Histogram({ name: 'node_yonderbox_graphql_api_request_buckets_milliseconds', help: 'api request duration buckets in milliseconds. Bucket size set to 500 and 2000 ms to enable apdex calculations with a T of 300ms', labelNames: ['url', 'method', 'status'], buckets: [ 500, 2000 ] })
    }
  },
  datasource: {
    requests: {
      duration: new client.Summary({ name: 'node_perfana_graphql_datasource_request_duration_milliseconds', help: 'datasource request duration in milliseconds', labelNames: ['datasource','status'] }),
      buckets: new client.Histogram({ name: 'node_perfana_graphql_datasource_request_buckets_milliseconds', help: 'datasource request duration buckets in milliseconds. Bucket size set to 500 and 2000 ms to enable apdex calculations with a T of 300ms', labelNames: ['datasource','status'], buckets: [ 500, 2000 ] })
    }
  },
}

// function report_backside(method, url, statusCode, start) {
//   const duration = util.duration(start)
//   if (typeof url === 'string') {
//     const parsed = URL.parse(url)
//     let reported = false
//     for (let name in config.dataSources) {
//       if (config.dataSources[name] && config.dataSources[name].enabled && config.dataSources[name].pattern) {
//         const pattern = config.dataSources[name].pattern
//         let match = true
//         for (let k in pattern) {
//           if (parsed[k] && !String(parsed[k]).match(pattern[k])) {
//             match = false
//           }
//         }
//         if (match) {
//           metric.datasource.requests.duration.labels(name, statusCode).observe(duration)
//           metric.datasource.requests.buckets.labels(name, statusCode).observe(duration)
//           reported = true
//         }
//       }
//     }
//     if (!reported) {
//       metric.api.requests.duration.labels(url, method || 'GET', statusCode).observe(duration)
//       metric.api.requests.buckets.labels(url, method || 'GET', statusCode).observe(duration)
//     }
//   }
// }

// function wrapper(protocol)
// {
//   const originalRequest = protocol.request
//   return function(req) {
//     const start = util.duration()
//     const result = originalRequest.apply(this, arguments)
//     result.on('error', (e) => {
//       console.error(`problem with request: ${e.message}`)
//     })
//     result.on('response', (res) => {
//       debugMetrics('request %y',{method:req.method, url:req.url || req.href, statusCode:res.statusCode, duration:util.duration(start)})
//       report_backside(req.method, req.href, res.statusCode, start)
//     })
//     return result
//   }
// }

// ********************************************

function report_frontside(method, url, statusCode, start) {
  const duration = util.duration(start)
  if (typeof url === 'string') {
    metric.web.requests.duration.labels(url, method || 'GET', statusCode).observe(duration)
    metric.web.requests.buckets.labels(url, method || 'GET', statusCode).observe(duration)
  }
}

function middleware (req, res, done) {
  const start = util.duration()

  res.on('finish', function () {
    // if (config.metrics.ignore.indexOf(req.path) < 0) {
      if (res.report) debugReport('%y',sortobject(res.report))
      debugReport('duration %y ms',util.duration(start))
      report_frontside(req.method || 'GET', req.path, res.statusCode, start)
    // }
  })

  return done()
}

function instrument(server, options) {
  // if (config.metrics.enabled) {
    server.use(middleware)
    server.get((options && options.url) || '/metrics', (req, res) => {
      res.header('content-type', 'text/plain; charset=utf-8')
      return res.send(client.register.metrics())
    })
  // }
}


module.exports = {
  instrument,
}
